==> edx_enterprise_data-8.9.0.tar.gz: edx_enterprise_data-8.9.0/enterprise_data/api/v1/serializers.py <==
"""
Serializers for enterprise api v1.
"""
from uuid import UUID

from rest_framework import serializers

from enterprise_data.admin_analytics.constants import Calculation, EnrollmentChart, Granularity, ResponseType
from enterprise_data.models import (
    EnterpriseAdminLearnerProgress,
    EnterpriseAdminSummarizeInsights,
    EnterpriseExecEdLCModulePerformance,
    EnterpriseLearner,
    EnterpriseLearnerEnrollment,
    EnterpriseOffer,
    EnterpriseSubsidyBudget,
)


class EnterpriseLearnerEnrollmentSerializer(serializers.ModelSerializer):
    """
    Serializer for EnterpriseLearnerEnrollment model.
    """
    course_api_url = serializers.SerializerMethodField()
    enterprise_user_id = serializers.SerializerMethodField()
    total_learning_time_hours = serializers.SerializerMethodField()

    class Meta:
        model = EnterpriseLearnerEnrollment
        # Do not change the order of fields below. Ordering is important becuase `progress_v3`
        # csv generated in `enterprise_reporting` should be same as csv generated on `admin-portal`
        # Order and field names below should match with `EnrollmentsCSVRenderer.header`
        fields = (
            'enrollment_id', 'enterprise_enrollment_id', 'is_consent_granted', 'paid_by',
            'user_current_enrollment_mode', 'enrollment_date', 'unenrollment_date',
            'unenrollment_end_within_date', 'is_refunded', 'seat_delivery_method',
            'offer_id', 'offer_name', 'offer_type', 'coupon_code', 'coupon_name', 'contract_id',
            'course_list_price', 'amount_learner_paid', 'course_key', 'courserun_key',
            'course_title', 'course_pacing_type', 'course_start_date', 'course_end_date',
            'course_duration_weeks', 'course_max_effort', 'course_min_effort',
            'course_primary_program', 'primary_program_type', 'course_primary_subject', 'has_passed',
            'last_activity_date', 'progress_status', 'passed_date', 'current_grade',
            'letter_grade', 'enterprise_user_id', 'user_email', 'user_account_creation_date',
            'user_country_code', 'user_username', 'enterprise_name', 'enterprise_customer_uuid',
            'enterprise_sso_uid', 'created', 'course_api_url', 'total_learning_time_hours', 'is_subsidy',
            'course_product_line', 'budget_id', 'enterprise_group_name', 'enterprise_group_uuid',
        )

    def get_course_api_url(self, obj):
        """Constructs course api url"""
        return '/enterprise/v1/enterprise-catalogs/{enterprise_customer_uuid}/courses/{courserun_key}'.format(
            enterprise_customer_uuid=obj.enterprise_customer_uuid, courserun_key=obj.courserun_key
        )

    def get_enterprise_user_id(self, obj):
        """Returns enterprise user id of a learner's enrollment"""
        return obj.enterprise_user_id

    def get_total_learning_time_hours(self, obj):
        """Returns the learners total learning time in hours"""
        return round((obj.total_learning_time_seconds or 0.0)/3600.0, 2)


class EnterpriseSubsidyBudgetSerializer(serializers.ModelSerializer):
    """
    Serializer for EnterpriseSubsidyBudget model.
    """

    class Meta:
        model = EnterpriseSubsidyBudget
        exclude = ('id',)


class EnterpriseOfferSerializer(serializers.ModelSerializer):
    """
    Serializer for EnterpriseOfferSerializer model.
    """
    budgets = serializers.SerializerMethodField()

    def get_budgets(self, instance):
        """Gets related budgets"""

        try:
            # if offer_id is a uuid
            offer_uuid = UUID(instance.offer_id, version=4)
            budget_queryset = EnterpriseSubsidyBudget.objects.filter(subsidy_uuid=offer_uuid)
            serializer = EnterpriseSubsidyBudgetSerializer(instance=budget_queryset, many=True, read_only=True)
            return serializer.data
        except ValueError:
            pass

        return []

    class Meta:
        model = EnterpriseOffer
        fields = '__all__'

    def to_internal_value(self, data):
        """
        Convert the incoming data offer_id field to a format that can be stored in the db.

        For a given offer_id string from the requester, determine the best representation to use for db storage.

        Raises serializers.ValidationError:
            If the given string is not exclusively numeric characters, but also does not parse as a UUID (either because
            it has the wrong length, incorrect dashes, or some other reason).
        """
        ret = super().to_internal_value(data)
        if ret['offer_id'] is None or ret['offer_id'] == '':
            raise serializers.ValidationError("requested offer_id is None.")

        if isinstance(ret['offer_id'], str) and len(ret['offer_id']) == 36:
            offer_id = ret['offer_id'].replace('-', '')
            # There should only be 4 dashes in the UUID, making the length 32 after removal
            if len(offer_id) == 32:
                ret['offer_id'] = offer_id
                return ret

            else:
                raise serializers.ValidationError("requested offer_id neither a valid integer nor UUID.")

        if len(ret['offer_id']) < 10:  # All ecommerce offer_ids are at < 1 million.
            try:
                int(ret['offer_id'])
                return ret
            except ValueError as e:
                raise serializers.ValidationError("Requested offer_id not a valid integer.") from e

        raise serializers.ValidationError("requested offer_id neither a valid integer nor UUID.")

    def to_representation(self, instance):
        """
        Add `-` dashes to the outgoing data offer_id field.
        """
        ret = super().to_representation(instance)

        # A 32 character offer_id is our heuristic for whether the stored value represents a UUID or integer.  If the
        # heuristic passes, make the serialized output look like a UUID.
        if len(ret['offer_id']) == 32:
            ret['offer_id'] = '-'.join([
                    ret['offer_id'][:8],
                    ret['offer_id'][8:12],
                    ret['offer_id'][12:16],
                    ret['offer_id'][16:20],
                    ret['offer_id'][20:]
                ]
            )

        return ret


class EnterpriseLearnerSerializer(serializers.ModelSerializer):
    """
    Serializer for EnterpriseLearner model.
    """

    class Meta:
        model = EnterpriseLearner
        fields = '__all__'

    def to_representation(self, instance):
        representation = super().to_representation(instance)

        if hasattr(instance, 'enrollment_count'):
            representation['enrollment_count'] = instance.enrollment_count
        if hasattr(instance, 'course_completion_count'):
            representation['course_completion_count'] = instance.course_completion_count

        return representation


class LearnerCompletedCoursesSerializer(serializers.Serializer):    # pylint: disable=abstract-method
    """
    Serializer for learner's completed courses.
    """
    class Meta:
        ref_name = 'v1.LearnerCompletedCoursesSerializer'

    user_email = serializers.EmailField()
    completed_courses = serializers.IntegerField()


class EnterpriseAdminLearnerProgressSerializer(serializers.ModelSerializer):
    """
    Serializer for EnterpriseAdminLearnerProgress model.
    """

    class Meta:
        model = EnterpriseAdminLearnerProgress
        fields = '__all__'


class EnterpriseAdminSummarizeInsightsSerializer(serializers.ModelSerializer):
    """
    Serializer for EnterpriseAdminSummarizeInsights model.
    """

    class Meta:
        model = EnterpriseAdminSummarizeInsights
        fields = '__all__'


class AdminAnalyticsAggregatesQueryParamsSerializer(serializers.Serializer):  # pylint: disable=abstract-method
    """
    Serializer for validating admin analytics query params.
    """
    start_date = serializers.DateField(required=False)
    end_date = serializers.DateField(required=False)
    granularity = serializers.CharField(required=False)
    calculation = serializers.CharField(required=False)
    response_type = serializers.CharField(required=False)
    page = serializers.IntegerField(required=False)
    chart_type = serializers.CharField(required=False)

    def validate(self, attrs):
        """
        Validate the query params.

        Raises:
            serializers.ValidationError: If start_date is greater than end_date.
        """
        if 'start_date' in attrs and 'end_date' in attrs:
            if attrs['start_date'] > attrs['end_date']:
                raise serializers.ValidationError("start_date should be less than or equal to end_date.")
        return attrs


class EnterpriseExecEdLCModulePerformanceSerializer(serializers.ModelSerializer):
    """
    Serializer for EnterpriseExecEdLCModulePerformance model.
    """

    class Meta:
        model = EnterpriseExecEdLCModulePerformance
        fields = '__all__'


class AdvanceAnalyticsQueryParamSerializer(serializers.Serializer):  # pylint: disable=abstract-method
    """Serializer for validating query params"""
    RESPONSE_TYPES = [
        ResponseType.JSON.value,
        ResponseType.CSV.value
    ]
    GRANULARITY_CHOICES = [
        Granularity.DAILY.value,
        Granularity.WEEKLY.value,
        Granularity.MONTHLY.value,
        Granularity.QUARTERLY.value
    ]
    CALCULATION_CHOICES = [
        Calculation.TOTAL.value,
        Calculation.RUNNING_TOTAL.value,
        Calculation.MOVING_AVERAGE_3_PERIOD.value,
        Calculation.MOVING_AVERAGE_7_PERIOD.value
    ]

    start_date = serializers.DateField(required=False)
    end_date = serializers.DateField(required=False)
    granularity = serializers.CharField(required=False)
    calculation = serializers.CharField(required=False)
    response_type = serializers.CharField(required=False)

    def validate(self, attrs):
        """
        Validate the query params.

        Raises:
            serializers.ValidationError: If start_date is greater than end_date.
        """
        start_date = attrs.get('start_date')
        end_date = attrs.get('end_date')

        if start_date and end_date and start_date > end_date:
            raise serializers.ValidationError("start_date should be less than or equal to end_date.")

        return attrs

    def validate_response_type(self, value):
        """
        Validate the response_type value.

        Raises:
            serializers.ValidationError: If response_type is not one of the valid choices in `RESPONSE_TYPES`.
        """
        if value not in self.RESPONSE_TYPES:
            raise serializers.ValidationError(f"response_type must be one of {self.RESPONSE_TYPES}")
        return value

    def validate_granularity(self, value):
        """
        Validate the granularity value.

        Raises:
            serializers.ValidationError: If granularity is not one of the valid choices.
        """
        if value not in self.GRANULARITY_CHOICES:
            raise serializers.ValidationError(f"Granularity must be one of {self.GRANULARITY_CHOICES}")
        return value

    def validate_calculation(self, value):
        """
        Validate the calculation value.

        Raises:
            serializers.ValidationError: If calculation is not one of the valid choices
        """
        if value not in self.CALCULATION_CHOICES:
            raise serializers.ValidationError(f"Calculation must be one of {self.CALCULATION_CHOICES}")
        return value


class AdvanceAnalyticsEnrollmentStatsSerializer(
    AdvanceAnalyticsQueryParamSerializer
):  # pylint: disable=abstract-method
    """Serializer for validating Advance Analytics Enrollments Stats API"""
    CHART_TYPES = [
        EnrollmentChart.ENROLLMENTS_OVER_TIME.value,
        EnrollmentChart.TOP_COURSES_BY_ENROLLMENTS.value,
        EnrollmentChart.TOP_SUBJECTS_BY_ENROLLMENTS.value
    ]

    chart_type = serializers.CharField(required=False)

    def validate_chart_type(self, value):
        """
        Validate the chart_type value.

        Raises:
            serializers.ValidationError: If chart_type is not one of the valid choices
        """
        if value not in self.CHART_TYPES:
            raise serializers.ValidationError(f"chart_type must be one of {self.CHART_TYPES}")
        return value

==> edx_enterprise_data-8.10.0.tar.gz: edx_enterprise_data-8.10.0/enterprise_data/api/v1/serializers.py <==
"""
Serializers for enterprise api v1.
"""
from uuid import UUID

from rest_framework import serializers

from enterprise_data.admin_analytics.constants import (
    Calculation,
    EngagementChart,
    EnrollmentChart,
    Granularity,
    ResponseType,
)
from enterprise_data.models import (
    EnterpriseAdminLearnerProgress,
    EnterpriseAdminSummarizeInsights,
    EnterpriseExecEdLCModulePerformance,
    EnterpriseLearner,
    EnterpriseLearnerEnrollment,
    EnterpriseOffer,
    EnterpriseSubsidyBudget,
)


class EnterpriseLearnerEnrollmentSerializer(serializers.ModelSerializer):
    """
    Serializer for EnterpriseLearnerEnrollment model.
    """
    course_api_url = serializers.SerializerMethodField()
    enterprise_user_id = serializers.SerializerMethodField()
    total_learning_time_hours = serializers.SerializerMethodField()

    class Meta:
        model = EnterpriseLearnerEnrollment
        # Do not change the order of fields below. Ordering is important becuase `progress_v3`
        # csv generated in `enterprise_reporting` should be same as csv generated on `admin-portal`
        # Order and field names below should match with `EnrollmentsCSVRenderer.header`
        fields = (
            'enrollment_id', 'enterprise_enrollment_id', 'is_consent_granted', 'paid_by',
            'user_current_enrollment_mode', 'enrollment_date', 'unenrollment_date',
            'unenrollment_end_within_date', 'is_refunded', 'seat_delivery_method',
            'offer_id', 'offer_name', 'offer_type', 'coupon_code', 'coupon_name', 'contract_id',
            'course_list_price', 'amount_learner_paid', 'course_key', 'courserun_key',
            'course_title', 'course_pacing_type', 'course_start_date', 'course_end_date',
            'course_duration_weeks', 'course_max_effort', 'course_min_effort',
            'course_primary_program', 'primary_program_type', 'course_primary_subject', 'has_passed',
            'last_activity_date', 'progress_status', 'passed_date', 'current_grade',
            'letter_grade', 'enterprise_user_id', 'user_email', 'user_account_creation_date',
            'user_country_code', 'user_username', 'enterprise_name', 'enterprise_customer_uuid',
            'enterprise_sso_uid', 'created', 'course_api_url', 'total_learning_time_hours', 'is_subsidy',
            'course_product_line', 'budget_id', 'enterprise_group_name', 'enterprise_group_uuid',
        )

    def get_course_api_url(self, obj):
        """Constructs course api url"""
        return '/enterprise/v1/enterprise-catalogs/{enterprise_customer_uuid}/courses/{courserun_key}'.format(
            enterprise_customer_uuid=obj.enterprise_customer_uuid, courserun_key=obj.courserun_key
        )

    def get_enterprise_user_id(self, obj):
        """Returns enterprise user id of a learner's enrollment"""
        return obj.enterprise_user_id

    def get_total_learning_time_hours(self, obj):
        """Returns the learners total learning time in hours"""
        return round((obj.total_learning_time_seconds or 0.0)/3600.0, 2)


class EnterpriseSubsidyBudgetSerializer(serializers.ModelSerializer):
    """
    Serializer for EnterpriseSubsidyBudget model.
    """

    class Meta:
        model = EnterpriseSubsidyBudget
        exclude = ('id',)


class EnterpriseOfferSerializer(serializers.ModelSerializer):
    """
    Serializer for EnterpriseOfferSerializer model.
    """
    budgets = serializers.SerializerMethodField()

    def get_budgets(self, instance):
        """Gets related budgets"""

        try:
            # if offer_id is a uuid
            offer_uuid = UUID(instance.offer_id, version=4)
            budget_queryset = EnterpriseSubsidyBudget.objects.filter(subsidy_uuid=offer_uuid)
            serializer = EnterpriseSubsidyBudgetSerializer(instance=budget_queryset, many=True, read_only=True)
            return serializer.data
        except ValueError:
            pass

        return []

    class Meta:
        model = EnterpriseOffer
        fields = '__all__'

    def to_internal_value(self, data):
        """
        Convert the incoming data offer_id field to a format that can be stored in the db.

        For a given offer_id string from the requester, determine the best representation to use for db storage.

        Raises serializers.ValidationError:
            If the given string is not exclusively numeric characters, but also does not parse as a UUID (either because
            it has the wrong length, incorrect dashes, or some other reason).
        """
        ret = super().to_internal_value(data)
        if ret['offer_id'] is None or ret['offer_id'] == '':
            raise serializers.ValidationError("requested offer_id is None.")

        if isinstance(ret['offer_id'], str) and len(ret['offer_id']) == 36:
            offer_id = ret['offer_id'].replace('-', '')
            # There should only be 4 dashes in the UUID, making the length 32 after removal
            if len(offer_id) == 32:
                ret['offer_id'] = offer_id
                return ret

            else:
                raise serializers.ValidationError("requested offer_id neither a valid integer nor UUID.")

        if len(ret['offer_id']) < 10:  # All ecommerce offer_ids are at < 1 million.
            try:
                int(ret['offer_id'])
                return ret
            except ValueError as e:
                raise serializers.ValidationError("Requested offer_id not a valid integer.") from e

        raise serializers.ValidationError("requested offer_id neither a valid integer nor UUID.")

    def to_representation(self, instance):
        """
        Add `-` dashes to the outgoing data offer_id field.
        """
        ret = super().to_representation(instance)

        # A 32 character offer_id is our heuristic for whether the stored value represents a UUID or integer.  If the
        # heuristic passes, make the serialized output look like a UUID.
        if len(ret['offer_id']) == 32:
            ret['offer_id'] = '-'.join([
                    ret['offer_id'][:8],
                    ret['offer_id'][8:12],
                    ret['offer_id'][12:16],
                    ret['offer_id'][16:20],
                    ret['offer_id'][20:]
                ]
            )

        return ret


class EnterpriseLearnerSerializer(serializers.ModelSerializer):
    """
    Serializer for EnterpriseLearner model.
    """

    class Meta:
        model = EnterpriseLearner
        fields = '__all__'

    def to_representation(self, instance):
        representation = super().to_representation(instance)

        if hasattr(instance, 'enrollment_count'):
            representation['enrollment_count'] = instance.enrollment_count
        if hasattr(instance, 'course_completion_count'):
            representation['course_completion_count'] = instance.course_completion_count

        return representation


class LearnerCompletedCoursesSerializer(serializers.Serializer):    # pylint: disable=abstract-method
    """
    Serializer for learner's completed courses.
    """
    class Meta:
        ref_name = 'v1.LearnerCompletedCoursesSerializer'

    user_email = serializers.EmailField()
    completed_courses = serializers.IntegerField()


class EnterpriseAdminLearnerProgressSerializer(serializers.ModelSerializer):
    """
    Serializer for EnterpriseAdminLearnerProgress model.
    """

    class Meta:
        model = EnterpriseAdminLearnerProgress
        fields = '__all__'


class EnterpriseAdminSummarizeInsightsSerializer(serializers.ModelSerializer):
    """
    Serializer for EnterpriseAdminSummarizeInsights model.
    """

    class Meta:
        model = EnterpriseAdminSummarizeInsights
        fields = '__all__'


class AdminAnalyticsAggregatesQueryParamsSerializer(serializers.Serializer):  # pylint: disable=abstract-method
    """
    Serializer for validating admin analytics query params.
    """
    start_date = serializers.DateField(required=False)
    end_date = serializers.DateField(required=False)
    granularity = serializers.CharField(required=False)
    calculation = serializers.CharField(required=False)
    response_type = serializers.CharField(required=False)
    page = serializers.IntegerField(required=False)
    chart_type = serializers.CharField(required=False)

    def validate(self, attrs):
        """
        Validate the query params.

        Raises:
            serializers.ValidationError: If start_date is greater than end_date.
        """
        if 'start_date' in attrs and 'end_date' in attrs:
            if attrs['start_date'] > attrs['end_date']:
                raise serializers.ValidationError("start_date should be less than or equal to end_date.")
        return attrs


class EnterpriseExecEdLCModulePerformanceSerializer(serializers.ModelSerializer):
    """
    Serializer for EnterpriseExecEdLCModulePerformance model.
    """

    class Meta:
        model = EnterpriseExecEdLCModulePerformance
        fields = '__all__'


class AdvanceAnalyticsQueryParamSerializer(serializers.Serializer):  # pylint: disable=abstract-method
    """Serializer for validating query params"""
    RESPONSE_TYPES = [
        ResponseType.JSON.value,
        ResponseType.CSV.value
    ]
    GRANULARITY_CHOICES = [
        Granularity.DAILY.value,
        Granularity.WEEKLY.value,
        Granularity.MONTHLY.value,
        Granularity.QUARTERLY.value
    ]
    CALCULATION_CHOICES = [
        Calculation.TOTAL.value,
        Calculation.RUNNING_TOTAL.value,
        Calculation.MOVING_AVERAGE_3_PERIOD.value,
        Calculation.MOVING_AVERAGE_7_PERIOD.value
    ]

    start_date = serializers.DateField(required=False)
    end_date = serializers.DateField(required=False)
    granularity = serializers.CharField(required=False)
    calculation = serializers.CharField(required=False)
    response_type = serializers.CharField(required=False)

    def validate(self, attrs):
        """
        Validate the query params.

        Raises:
            serializers.ValidationError: If start_date is greater than end_date.
        """
        start_date = attrs.get('start_date')
        end_date = attrs.get('end_date')

        if start_date and end_date and start_date > end_date:
            raise serializers.ValidationError("start_date should be less than or equal to end_date.")

        return attrs

    def validate_response_type(self, value):
        """
        Validate the response_type value.

        Raises:
            serializers.ValidationError: If response_type is not one of the valid choices in `RESPONSE_TYPES`.
        """
        if value not in self.RESPONSE_TYPES:
            raise serializers.ValidationError(f"response_type must be one of {self.RESPONSE_TYPES}")
        return value

    def validate_granularity(self, value):
        """
        Validate the granularity value.

        Raises:
            serializers.ValidationError: If granularity is not one of the valid choices.
        """
        if value not in self.GRANULARITY_CHOICES:
            raise serializers.ValidationError(f"Granularity must be one of {self.GRANULARITY_CHOICES}")
        return value

    def validate_calculation(self, value):
        """
        Validate the calculation value.

        Raises:
            serializers.ValidationError: If calculation is not one of the valid choices
        """
        if value not in self.CALCULATION_CHOICES:
            raise serializers.ValidationError(f"Calculation must be one of {self.CALCULATION_CHOICES}")
        return value


class AdvanceAnalyticsEnrollmentStatsSerializer(
    AdvanceAnalyticsQueryParamSerializer
):  # pylint: disable=abstract-method
    """Serializer for validating Advance Analytics Enrollments Stats API"""
    CHART_TYPES = [
        EnrollmentChart.ENROLLMENTS_OVER_TIME.value,
        EnrollmentChart.TOP_COURSES_BY_ENROLLMENTS.value,
        EnrollmentChart.TOP_SUBJECTS_BY_ENROLLMENTS.value
    ]

    chart_type = serializers.CharField(required=False)

    def validate_chart_type(self, value):
        """
        Validate the chart_type value.

        Raises:
            serializers.ValidationError: If chart_type is not one of the valid choices
        """
        if value not in self.CHART_TYPES:
            raise serializers.ValidationError(f"chart_type must be one of {self.CHART_TYPES}")
        return value


class AdvanceAnalyticsEngagementStatsSerializer(
    AdvanceAnalyticsQueryParamSerializer
):  # pylint: disable=abstract-method
    """Serializer for validating Advance Analytics Engagements Stats API"""
    CHART_TYPES = [
        EngagementChart.ENGAGEMENTS_OVER_TIME.value,
        EngagementChart.TOP_COURSES_BY_ENGAGEMENTS.value,
        EngagementChart.TOP_SUBJECTS_BY_ENGAGEMENTS.value
    ]

    chart_type = serializers.CharField(required=False)

    def validate_chart_type(self, value):
        """
        Validate the chart_type value.

        Raises:
            serializers.ValidationError: If chart_type is not one of the valid choices
        """
        if value not in self.CHART_TYPES:
            raise serializers.ValidationError(f"chart_type must be one of {self.CHART_TYPES}")
        return value
